package at.ac.uibk.gitsearch.service;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

import org.codeability.sharing.plugins.api.ShoppingBasket;
import org.codeability.sharing.plugins.api.ShoppingBasket.ExerciseInfo;
import org.codeability.sharing.plugins.api.ShoppingBasket.UserInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;

/**
 * Service for exercise/course search results
 * <p>
 */
@Service
public class ShoppingBasketService {
	
	@Autowired
	private PluginManagementService pluginManagementService;
	
	public static class ShoppingBasketRedirectInfoDTO {
		private String redirectURL;
		private String shoppingBasketToken;
		private String callBackURL;
		/**
		 * @return the redirectURL
		 */
		public String getRedirectURL() {
			return redirectURL;
		}
		/**
		 * @param redirectURL the redirectURL to set
		 */
		public void setRedirectURL(String redirectURL) {
			this.redirectURL = redirectURL;
		}
		/**
		 * @return the shoppingBasketToken
		 */
		public String getShoppingBasketToken() {
			return shoppingBasketToken;
		}
		/**
		 * @param shoppingBasketToken the shoppingBasketToken to set
		 */
		public void setShoppingBasketToken(String shoppingBasketToken) {
			this.shoppingBasketToken = shoppingBasketToken;
		}
		/**
		 * @return the callBackURL
		 */
		public String getCallBackURL() {
			return callBackURL;
		}
		/**
		 * @param callBackURL the callBackURL to set
		 */
		public void setCallBackURL(String callBackURL) {
			this.callBackURL = callBackURL;
		}
	}
	
	public static class ShoppingBasketInfoDTO {
		private ItemInfoDTO[] itemInfos;
		private String plugin;
		private String action;
		
		public ShoppingBasketInfoDTO(ItemInfoDTO[] itemInfos) {
			super();
			this.itemInfos = itemInfos;
		}
		
		public ShoppingBasketInfoDTO() {
			// JSON
		}
		
		

		/**
		 * @return the itemInfos
		 */
		public ItemInfoDTO[] getItemInfos() {
			return itemInfos;
		}

		/**
		 * @param itemInfos the itemInfos to set
		 */
		public void setItemInfos(ItemInfoDTO[] itemInfos) {
			this.itemInfos = itemInfos;
		}



		/**
		 * @return the plugin
		 */
		public String getPlugin() {
			return plugin;
		}

		/**
		 * @param plugin the plugin to set
		 */
		public void setPlugin(String plugin) {
			this.plugin = plugin;
		}

		/**
		 * @return the action
		 */
		public String getAction() {
			return action;
		}

		/**
		 * @param action the action to set
		 */
		public void setAction(String action) {
			this.action = action;
		}



		public static class ItemInfoDTO {
		String exerciseIDs;
		String gitLabURls;
		
		public ItemInfoDTO(String exerciseIDs, String gitLabURls) {
			super();
			this.exerciseIDs = exerciseIDs;
			this.gitLabURls = gitLabURls;
		}
		
		public ItemInfoDTO() {
			// JSON
		}
		/**
		 * @return the exerciseIDs
		 */
		public String getExerciseIDs() {
			return exerciseIDs;
		}
		/**
		 * @param exerciseIDs the exerciseIDs to set
		 */
		public void setExerciseIDs(String exerciseIDs) {
			this.exerciseIDs = exerciseIDs;
		}
		/**
		 * @return the gitLabURls
		 */
		public String getGitLabURls() {
			return gitLabURls;
		}
		/**
		 * @param gitLabURls the gitLabURls to set
		 */
		public void setGitLabURls(String gitLabURls) {
			this.gitLabURls = gitLabURls;
		}
		}
	}
	

	private final Logger log = LoggerFactory.getLogger(ShoppingBasketService.class);


	public ShoppingBasketService() {

	}
	
	public ShoppingBasket getBasket(String basketToken) {
		final UserInfo userInfo = new UserInfo("Michael.Breu@uibk.ac.at");
		final ExerciseInfo exerciseInfo = new ExerciseInfo("JUnit Quality Tests Exercise", 3, "https://sharing.codeability-austria.uibk.ac.at/sharing/university-innsbruck/java/general/junit-quality-tests-exercise.git");
		final ShoppingBasket shoppingBasket = new ShoppingBasket(userInfo, new ExerciseInfo[] {exerciseInfo});
		return shoppingBasket;
	}
	
	LoadingCache<String, ShoppingBasketInfoDTO> basketCache = CacheBuilder.newBuilder()
		    .maximumSize(10000)
		    .expireAfterWrite(10, TimeUnit.MINUTES)
		    .build(
		        new CacheLoader<String, ShoppingBasketInfoDTO>() {
		          public ShoppingBasketInfoDTO load(String key)  {
		        	  return new ShoppingBasketInfoDTO();
		          }
		        });
	public ShoppingBasketRedirectInfoDTO getRedirectInfo(ShoppingBasketInfoDTO basket, String baseURL) {
		UUID random = UUID.randomUUID();
		basketCache.put(random.toString(), basket);
		
		ShoppingBasketRedirectInfoDTO result = new ShoppingBasketRedirectInfoDTO();
		result.setRedirectURL(pluginManagementService.getRedirectURL(basket.getPlugin(), basket.getAction()));
		result.setShoppingBasketToken(random.toString());
		result.setCallBackURL(baseURL);
		
		return result;
		
	}

}
